package org.jeecg.modules.adweb.dmp.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import io.swagger.v3.oas.annotations.tags.Tag;

import lombok.extern.slf4j.Slf4j;

import org.apache.commons.lang3.tuple.Pair;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.adweb.dmp.dto.google.analytics.GAAccountDTO;
import org.jeecg.modules.adweb.dmp.entity.GoogleGTM;
import org.jeecg.modules.adweb.dmp.service.IGoogleGTMService;
import org.jeecg.modules.adweb.dmp.service.google.GTMAdminService;
import org.jeecg.modules.adweb.site.entity.AdwebSite;
import org.jeecg.modules.adweb.site.service.IAdwebSiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Objects;

/**
 * GTM容器和标签管理
 *
 * @author wfansh
 */
@Tag(name = "Google Tag Manager")
@RestController
@RequestMapping("/gtm")
@Slf4j
public class GTMController {
    @Autowired private IAdwebSiteService adwebSiteService;
    @Autowired private IGoogleGTMService googleGTMService;
    @Autowired private GTMAdminService gtmAdminService;

    @RequestMapping(value = "/get", method = RequestMethod.GET)
    @ResponseBody
    public Result<AdwebSite> getGTMSnippets(String siteCode) {
        GoogleGTM googleGTM =
                googleGTMService.getOne(
                        new LambdaQueryWrapper<GoogleGTM>().eq(GoogleGTM::getSiteCode, siteCode));
        if (Objects.isNull(googleGTM)) {
            log.info("站点 {} 对应的GoogleGTM不存在", siteCode);
            return null;
        }

        AdwebSite adwebSite = new AdwebSite();
        Pair<String, String> snippets = gtmAdminService.getSnippets(googleGTM.getGtmTagId());
        adwebSite.setGtmHead(snippets.getKey());
        adwebSite.setGtmBody(snippets.getValue());

        return Result.ok(adwebSite);
    }

    @RequestMapping(value = "/create", method = RequestMethod.GET)
    @ResponseBody
    public Result<AdwebSite> createGTMSnippets(String siteCode, String gaAccountId) {
        AdwebSite adwebSite = adwebSiteService.getSiteByCode(siteCode);

        GoogleGTM googleGTM =
                gtmAdminService.createContainer(
                        adwebSite.getCode(),
                        adwebSite.getDomain(),
                        adwebSite.getName(),
                        gaAccountId);

        Pair<String, String> snippets = gtmAdminService.getSnippets(googleGTM.getGtmTagId());
        adwebSite.setGtmHead(snippets.getKey());
        adwebSite.setGtmBody(snippets.getValue());

        return Result.ok(adwebSite);
    }

    @RequestMapping(value = "/listGAAccounts", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<Pair<String, String>>> listGAAccounts() {
        List<GAAccountDTO> gaAccounts = gtmAdminService.listGAAccounts();

        return Result.ok(
                gaAccounts.stream()
                        .map(gaAccount -> Pair.of(gaAccount.getDisplayName(), gaAccount.getId()))
                        .toList());
    }
}
